#! /usr/bin/env python3

import sys
import yaml
import os
import mariadb

__author__ = "Diarmuid O'Briain"
__copyright__ = "Copyright 2024, Technological University of the Shannon"
__licence__ = "European Union Public Licence v1.2"
__version__ = "2.1"
__role__ = "Default Database creator"
__program__ = sys.argv[0][2:] if (sys.argv[0][:2] == "./") else sys.argv[0]
__dir__ = os.path.dirname(os.path.realpath(__program__))

title = "Default TUS Database generator"
conf = dict()
table = str()
db_tables = ("EngProject", "EngHobbies")

db_tables = {
    "EngProject": f"CREATE TABLE EngProject (Student_no INT NOT NULL, Username TEXT NULL, FirstName TEXT NULL, LastName TEXT NULL, Email TEXT NULL, Role TEXT NULL, PRIMARY KEY (Student_no));",
    "EngHobbies": f"CREATE TABLE EngHobbies (Student_no INT NOT NULL, Hobbies TEXT NULL, PRIMARY KEY (Student_no));",
}

db_data = {
    "EngProject": (
        "000000, 'cbabage', 'Charles', 'Babbage', 'charles@babbage.com', 'Hardware'",
        "000001, 'alovelace', 'Ada', 'Lovelace', 'ada@lovelace.com', 'Programmer'",
        "000002, 'lmenabrea', 'Luigi', 'Menabrea', 'luigi@menabrea.it', 'Politician'",
    ),
    "EngHobbies": (
        "000000, 'cricket, cards'",
        "000001, 'camogie, horses'",
        "000002, 'soccer, pasta'",
    ),
}

# // Header //
print()
print(title)
print(f"{'-' * len(title)}")
print()
print(
    f"""This program drops existing '{list(db_tables.keys())[0]}' and '{list(db_tables.keys())[1]}' 
tables from the 'Eng' database, creates new tables and   
populates them with some sample data."""
)

# // Get the configuration information from YAML //
with open(f"{__dir__}/conf.yml", "r") as fh:
    try:
        conf = yaml.safe_load(fh)
        table = conf.pop("table")
    except yaml.YAMLError as err:
        print(f"Error: {err}", file=sys.stderr)
        sys.exit(1)

# // Ask for password //
print()
conf["password"] = input("Enter the password to access database: ")

# // Connect to the MariaDB database //
try:
    conn = mariadb.connect(
        user=conf["user"],
        password=conf["password"],
        host=conf["host"],
        port=conf["port"],
        database=conf["database"],
    )
    cur = conn.cursor()
    print(f"\nConnected to the {conf['database']} database\n")

except mariadb.Error as e:
    print(f"\nError connecting to MariaDB Platform: {e}")
    sys.exit(1)

# // Delete the current database tables in MariaDB //
for key in db_tables.keys():
    query = f"DROP TABLE IF EXISTS Eng.{key};"
    cur.execute(query)
    print(query)

# // Create tables //
for query in db_tables.values():
    cur.execute(query)
    print(query)

# // Populate tables with some data //
for key, value in db_data.items():
    for str_ in value:
        query = f"INSERT INTO {key} VALUES ({str_})"
        cur.execute(query)
        conn.commit()
        print(query)

# // Close connection to MariaDB //
cur.close()
conn.close()

# // End //
print(f"\nDatabase {conf['database']} has now been populated.\n")
